{
'
'  Catalyst_Arguments - an object to allow a SPIN program to
'  retrieve a command line stored by Catalyst. It simulates
'  the kind of argc/argv processing used in C programs. 
'
'
'  To use this module (assuming it is called Arguments, and that
'  we have a function called Print) we could do something like:
'
'    VAR
'      long buffer[Arguments#MAX_LONGS]
'
'    PUB My_Program
'      Arguments.init (buffer)
'      if (Arguments.argc > 0)
'        repeat 
'          Print (Arguments.argv(i))
'
'
}

CON

' These values must match the ones defined in Catalina_CogStore.spin :

MAX_LONGS     = 300             ' store up to 300 longs (1200 chars)

COGSTORE      = $7EFC           ' LONG location monitored by this cog for commands

ARGC_ADDR     = $7F30           ' WORD argc goes here (will be loaded during initialization)

ARGV_ADDR     = $7F32           ' WORD address of argv must be stored here

ARGV_0        = $7F34           ' LONG default argv array starts here (up to ARGV_MAX entries)

ARGV_MAX      = 32              ' maximum size (in LONGs) of default argv array


CMD_READ   = $1000_0000         ' copy from Hub to Cog (address in lower 24 bits)
                                ' COGSTORE set to zero when complete

CMD_WRITE  = $2000_0000         ' copy from Cog to Hub (address in lower 24 bits)
                                ' COGSTORE set to zero when complete

CMD_SIZE   = $3000_0000         ' returns size of stored data (in LONGs) - set the
                                ' lower 24 bits to FFFFFF on call - lower 24 bits
                                ' COGSTORE set to size when complete

CMD_SETUP  = $4000_0000         ' setup argc and argv array with the stored data
                                ' COGSTORE set to zero when complete

CMD_RESPONSE = $FEED_FACE       ' COGSTORE set to this on any other command

QUOTE_CHAR = $22

PUB init (arguments)
   ' set up arguments taken from command line (if there were any)
   Setup (arguments)
   ' no longer need the arguments
   Stop

PUB argc : n
  ' return the count of arguments
  n := WORD[ARGC_ADDR]
 
PUB argv(n) : addr

  ' return a pointer to the nth argument
  addr := LONG[ARGV_0][n]
 
PRI Valid : ok

  long[COGSTORE] := -1 ' any invalid command
  repeat 100
    if long[COGSTORE] == CMD_RESPONSE
      return -1
  return 0

PRI Stop

  if Valid
    ' stop the COGSTORE  
    cogstop(7)

PRI Write(Addr) : ok

  if Valid
    long[COGSTORE] := CMD_WRITE | Addr
    repeat 100000
      if long[COGSTORE] == 0
        return -1
    return 0
  else
    return 0


PRI Read(Addr) : ok

  if Valid
    long[COGSTORE] := CMD_READ | Addr
    repeat 100000
      if long[COGSTORE] == 0
        return -1
    return 0
  else
    return 0


PRI Size | s

  if not Valid
    return -1
    
  long[COGSTORE] := CMD_SIZE | $FFFFFF 
  repeat 100
    if long[COGSTORE] <> CMD_SIZE | $FFFFFF
      return long[COGSTORE] & $FFFFFF
  return -2

PUB Setup(addr) | i

  word[ARGC_ADDR] := 0
  word[ARGV_ADDR] := 0
  repeat i from 0 to ARGV_MAX-1
    long[ARGV_0][i] := 0
    
  if not Valid
    word[ARGC_ADDR] := 1
    word[ARGV_ADDR] := ARGV_0
    ' use "null" as name if no arguments
    long[ARGV_0] := ARGV_0 + 4*(ARGV_MAX-2)
    long[ARGV_0][ARGV_MAX-2] := $6C6C756E ' null
  else
    long[COGSTORE] := CMD_SETUP | Addr
    repeat 100000
      if long[COGSTORE] == 0
        return -1
    return 0
      